'''
Created on Mar 26, 2010

@author: seif
'''
import time
import gobject
import os
import urllib

from zeitgeist.datamodel import StorageState, ResultType, Event, Subject, Interpretation, Manifestation
from zeitgeist.client import ZeitgeistClient
from xdg.Config import icon_size
from widgets.ui_utils import ITEM_CACHE
from base import Item, ItemSource

CLIENT = ZeitgeistClient()


def event_exists(uri):
    # TODO: Move this into Zeitgeist's datamodel.py
    if uri.startswith("trash://"):
        return False
    return not uri.startswith("file://") or os.path.exists(
        urllib.unquote(str(uri[7:])))


class Source(ItemSource):

    def __init__(self, interpretation, icon, desc_sing, icon_size):
        ItemSource.__init__(self, interpretation, desc_sing, icon)
        self.interpretation = interpretation
        self.name = desc_sing
        self.icon = icon
        self._desc_sing = desc_sing
        self.icon_size = icon_size
        
    def get_items_uncached(self, callback):
        
        def callback2(events):
            items = []
            for event in events:
                event = Event.new_for_struct(event)
                subject = event.subjects[0]
                if event_exists(subject.uri):
                    items.append(subject.uri)     
                    if not ITEM_CACHE.has_key(subject.uri):                 
                        item = Item(subject.uri, subject.text, icon=None, comment="", launcher=None, timestamp=int(event.timestamp)/1000, subject=subject, icon_size = self.icon_size)
                        ITEM_CACHE[subject.uri] = item
            callback(items)
        
        event = Event()
        subject = Subject()
        if self.interpretation:
            subject.set_interpretation(self.interpretation)
            event.set_subjects([subject])
        end = time.time()
        start = time.time() - self.zoom_level*24*60*60
        CLIENT.find_events_for_templates(
            [event],
            callback2,
            (start * 1000, end * 1000),
            StorageState.Any,
            0,
            self.sorting,            
        )
        

SUPPORTED_SOURCES = {
    # TODO: Please change this description stuff. To give a single reason,
    # saying "videos watched" is a lie because you may have edited them, or
    # it may just be a "bookmark event".
    #
    # Also, how this is implemented now won't work fine with i18n.
    Interpretation.VIDEO.uri: Source(Interpretation.VIDEO, "gnome-mime-video", "Video", 56),
    Interpretation.MUSIC.uri: Source(Interpretation.MUSIC, "gnome-mime-audio", "Audio", 48),
    Interpretation.IMAGE.uri: Source(Interpretation.IMAGE, "image", "Images", 64),
    Interpretation.DOCUMENT.uri: Source(Interpretation.DOCUMENT, "stock_new-presentation", "Documents", 96),
    Interpretation.SOURCECODE.uri: Source(Interpretation.SOURCECODE, "applications-development", "Development", 48),
    Interpretation.UNKNOWN.uri: Source(Interpretation.UNKNOWN, "applications-other", "Other", 48),
}


class MenuTree(gobject.GObject):
    __gsignals__ = {
        "reload" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ())
        }
    
    def __init__ (self):
        gobject.GObject.__init__(self)
        

    def get_toplevel_sources(self):
    
        sources = []
        source = Source(None, "misc", "All Files", 56)
        sources.append(source)
        sources.append("---")
        
        for source in SUPPORTED_SOURCES.values():
            sources.append(source)
        
        return sources

    

    
library = MenuTree()

