'''
Created on Mar 25, 2010

@author: seif
'''

import gtk
import gobject
import pango
import datetime
from ui_utils import icon_factory
from widgets.ui_utils import ITEM_CACHE


class Topic(gtk.ToggleButton):     
    def __init__(self, label):   
        gtk.ToggleButton.__init__(self, label)
        self.label = label
        self.set_relief(gtk.RELIEF_NONE)
        self.set_focus_on_click(False)
        self.categories = []
        

class LibraryTopic(Topic):
    def __init__(self):
        Topic.__init__(self, "Library")
        from datasources.library import library
        self.categories += library.get_toplevel_sources()
    
        
class TopicsWindow(gtk.Window):
    def __init__(self):
        gtk.Window.__init__(self)
        self.topics = [LibraryTopic()]
        self.set_size_request(640, 400)
        #self.set_decorated(False)
        self.set_border_width(3)
        
        self.notebook = gtk.Notebook()
        self.notebook.set_show_border(False)
        self.notebook.set_show_tabs(False)
        for topic in self.topics:
            topicview = TopicView(topic.categories)
            self.notebook.append_page(topicview, gtk.Label(topic.label))
            topic.connect("clicked", self._switch_window)
        self.add(self.notebook)
        self.connect("focus-out-event", self._hide_window)
        self.show_all()
        
    def _hide_window(self, window, event):
        #print window, event
        pass
        
    def _switch_window(self, widget):
        self.notebook.set_page(self.topics.index(widget))
        
class TopicView(gtk.VBox):
    def __init__(self, categories):
        gtk.VBox.__init__(self)
        
        self.search = SearchEntry()
        self.sidebar = Sidebar()
        
        hbox = gtk.HBox()
        hbox.pack_start(gtk.Label("Search: "), False, False)
        hbox.pack_start(self.search)
        self.pack_start(hbox, False, False, 6)
        self.pack_start(self.sidebar, False, False)
        
        self.categories = categories
        self.hbox = gtk.HBox()
        self.view = CategoryView()
        self.sidebar.set_view(self.view)
        #self.hbox.pack_start(self.sidebar, False, False)
        self.vbox = gtk.VBox()
        
        self.pack_start(self.hbox)
        scroll = gtk.ScrolledWindow()
        scroll.add_with_viewport(self.view)
        scroll.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        scroll.get_children()[0].set_shadow_type(gtk.SHADOW_NONE)

        topbar = TimeBar()
        ev = gtk.EventBox()
        ev.add(topbar)
        self.vbox.pack_start(ev, False, False)
        self.vbox.pack_start(gtk.HSeparator(), False, False)
        self.vbox.pack_start(scroll)   
        
        scroll =gtk.ScrolledWindow()
        scroll.add_with_viewport(self.vbox)        
        scroll.set_policy(gtk.POLICY_NEVER, gtk.POLICY_NEVER)
        self.hbox.pack_start(scroll)
        
        topbar.connect("zoom-changed", self._zoom_changed)
        topbar.connect("sorting-changed", self._sort_changed)
        
        for cat in categories:
            if isinstance(cat, str):
                self.sidebar.pack_start(gtk.HSeparator(), False, False)
            else:
                self.sidebar.append_category(cat)
        self.sidebar.connect("clicked", self.set_view)
        
        self.sidebar.buttons[0].set_active(True)
        
        def change_style(widget, style):
            pass

        self.view.connect("style-set", change_style)
        
    def _sort_changed(self, sender, sorting):
        self.sidebar.set_sorting(sorting)

    def _zoom_changed(self, sender, zoom_level):
        self.sidebar.set_zoom_level(zoom_level)
        
            
    def set_view(self, button, items):
        if len(items) > 0:
            item = items[0]
        
        self.view.clear()
        for item in items:
            self.view.add_item(item)
        #self.view.update_size(item.icon_size)
                
                    
class Sidebar(gtk.HBox): 
    __gsignals__ = {
        "clicked" : (gobject.SIGNAL_RUN_FIRST,
                    gobject.TYPE_NONE,
                    (gobject.TYPE_PYOBJECT,))
    }   
    def __init__(self):
        gtk.HBox.__init__(self)
        self.busy = False
        self.buttons = []
        
    def set_view(self, view):
        self.view = view
        
    def set_zoom_level(self, zoom_level):
        for button in self.buttons:
            button.category.set_zoom_level(zoom_level)
            if button.get_active():
                button.emit("toggled")
                
    def set_sorting(self, sorting):
        for button in self.buttons:
            button.category.set_sorting(sorting)
            if button.get_active():
                button.emit("toggled")
        
    def append_category(self, cat):
        
        def callback(button):
            def callback2(items):
                self.emit("clicked", items)
            button.get_items(callback2)
        
        btn = CategoryButton(cat)
        self.pack_start(btn, False, False)
        self.buttons.append(btn)
        btn.connect_after("toggled", callback)
        btn.connect("toggled", self.untoggle_all)
        
    def untoggle_all(self, btn):
        if not self.busy:
            self.busy = True
            for b in self.buttons:
                if not b == btn: 
                    b.set_active(False)
            self.busy = False
            
        
class CategoryButton(gtk.ToggleButton):
    def __init__(self, category):
        gtk.ToggleButton.__init__(self)
        self.category = category       
        #icon = icon_factory.load_icon(self.category.icon, 24)
        label = gtk.Label(category.title)
        hbox = gtk.HBox()
        #hbox.pack_start(gtk.image_new_from_pixbuf(icon), False, False)
        hbox.pack_start(label)
        self.add(hbox)
        self.set_relief(gtk.RELIEF_NONE)
        self.set_border_width(1)
        self.set_focus_on_click(False)
        #self.set_alignment(0.0, 0.5)
        
    def get_items(self, callback):
        self.category.get_items(callback)
        
class CategoryView(gtk.IconView):
    def __init__(self):
        gtk.IconView.__init__(self)
        self.model = gtk.ListStore(gtk.gdk.Pixbuf, str, gobject.TYPE_PYOBJECT)
        self.set_margin(12)
        self.use_cells = isinstance(self, gtk.CellLayout)
        self.set_spacing(4)
        self.set_selection_mode(gtk.SELECTION_MULTIPLE)
        self.icon_cell = gtk.CellRendererPixbuf()
        self.icon_cell.set_property("xalign", 0.5)
        self.icon_cell.set_property("yalign", 1.0)
        self.icon_cell.set_fixed_size(128, -1)
        self.pack_start(self.icon_cell, expand=False)
        self.add_attribute(self.icon_cell, "pixbuf", 0)

        self.text_cell = gtk.CellRendererText()
        #self.text_cell.set_property("wrap-mode", pango.WRAP_WORD_CHAR)
        self.text_cell.set_property("yalign", 0.0)
        self.text_cell.set_property("xalign", 0.5)
        self.text_cell.set_property("ellipsize", pango.ELLIPSIZE_END)
        self.pack_start(self.text_cell, expand=True)
        self.add_attribute(self.text_cell, "markup", 1)
        #self.set_item_width(150)
        self.set_model(self.model)
        #self.set_orientation(gtk.ORIENTATION_HORIZONTAL)            
        #self.text_cell.set_property("wrap-width", 100)

        
    def clear(self):
        self.model.clear()
    
    def add_item(self, item):
        item = ITEM_CACHE[item]
        icon = item.get_icon()
        text = "<span>" + item.title + "</span>\n" + "<span size='small'>"+ item.comment + "</span>"
        text = text.replace("%20", " ")
        text = text.replace("%5B", "["); text = text.replace("%5D", "]")
        self.model.append([icon, text, item])
        
    def update_size(self, size):
        self._update_sizing(size)
        
    def _update_sizing(self, icon_width):
        pass
        #self.icon_cell.set_fixed_size(icon_width, -1) # Reset icon sizing
            #wrap_width = 150 - icon_width - 4
            #self.text_cell.set_property("wrap-width", wrap_width)
        
class TimeBar(gtk.EventBox):
    __gsignals__ = {
        "zoom-changed" : (gobject.SIGNAL_RUN_LAST,
                          gobject.TYPE_NONE,
                          (gobject.TYPE_INT,)),
        "sorting-changed" : (gobject.SIGNAL_RUN_LAST,
                          gobject.TYPE_NONE,
                          (gobject.TYPE_INT,)),
        }

    _zoomlist = [[_("Today"), 1],
                 [_("This week"), 7],
                 [_("Last 2 weeks"), 14],
                 [_("This month"), 30],
                 [_("Last 2 months"), 60],
                 [_("Last 3 months"), 90],
                 [_("Last 6 months"), 180]]
    _default_zoom = 7

    def __init__(self):
        gtk.EventBox.__init__(self)
        self.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
        
        self.sorting = 2
        
        box = gtk.HBox(False, 0)
        box.show()
        self.add(box)

        self._add_sparkline(box)

        self.out_btn = gtk.Button()
        self.out_btn.add(gtk.image_new_from_stock(gtk.STOCK_GO_FORWARD, gtk.ICON_SIZE_SMALL_TOOLBAR))
        self.out_btn.set_relief(gtk.RELIEF_NONE)
        self.out_btn.connect("clicked", lambda btn: self._zoom_clicked(1))
        box.pack_end(self.out_btn, False, False)
        
        self.in_btn = gtk.Button()
        self.in_btn.add(gtk.image_new_from_stock(gtk.STOCK_GO_BACK, gtk.ICON_SIZE_SMALL_TOOLBAR))
        self.in_btn.set_relief(gtk.RELIEF_NONE)
        self.in_btn.connect("clicked", lambda btn: self._zoom_clicked(-1))
        box.pack_end(self.in_btn, False, False)
        
        self.label = gtk.Label()
        self.label.set_width_chars(max([len(x[0]) for x in self._zoomlist]))
        self.label.set_alignment(1.0, 0.5)
        box.pack_end(self.label, False, False, 6)

        box.show_all()

        self.set_zoom_level(self._default_zoom)
        
        self.combobox = gtk.combo_box_entry_new_text()
        label = gtk.Label(" Sort By: ")
        hbox = gtk.HBox()
        hbox.pack_start(label, False, False)
        hbox.pack_start(self.combobox, False, False)
        box.pack_start(hbox, False, False)
        
        self.combobox.append_text("Recency")
        self.combobox.append_text("Popularity")
        self.combobox.set_active(0)
        
        self.combobox.connect("changed", self._sorting_changed)
        
    def _sorting_changed(self, widget):
        #print widget
        self.sorting = widget.get_active()
        
        if self.sorting == 0:
            self.sorting = 2
        else:
            self.sorting = 4
            
        self.emit("sorting-changed", self.sorting)

    def _add_sparkline(self, box):
        self.spark = None
        
        ### Disabled due to bugginess for 0.2.8
        #self.spark = SparkLine(self.get_zoom_level_list(floor=0))
        #self.spark.connect("zoom-changed", lambda w, num_days: self.set_zoom_level(num_days))

        #align = gtk.Alignment(0.5, 0.5, 1.0, 1.0)
        #align.set_property("top-padding", 4)
        #align.add(self.spark)
        #box.pack_start(align, True, True, 6)

    def get_zoom_level(self):
        return self.zoom_level

    def get_zoom_level_list(self, floor = _default_zoom):
        return [days for name, days in self._zoomlist if days >= floor]

    def _zoom_clicked(self, dir):
        for i in range(len(self._zoomlist)):
            label, days = self._zoomlist[i]
            if days == self.zoom_level:
                try:
                    label, days = self._zoomlist[i + dir]
                except IndexError:
                    label, days = self._zoomlist[0]
                self.set_zoom_level(days, label)
                break

    def set_zoom_level(self, zoom_level, label=None):
        self.zoom_level = zoom_level

        if not label:
            for _label, days in self._zoomlist:
                if days == zoom_level:
                    label = _label
        self.label.set_text(label)
        if self.spark:
            self.spark.set_zoom_level(zoom_level)
        self.emit("zoom-changed", zoom_level)

    def set_items(self, items):
        dates = [x.get_timestamp() for x in items if x.get_timestamp() != 0]
        dates = [datetime.date.fromtimestamp(x) for x in dates]
        if self.spark:
            self.spark.set_dates(dates)

class SearchEntry(gtk.Entry):

    __gsignals__ = {
        "clear" : (gobject.SIGNAL_RUN_FIRST,
                   gobject.TYPE_NONE,
                   ()),
        "search" : (gobject.SIGNAL_RUN_FIRST,
                    gobject.TYPE_NONE,
                    (gobject.TYPE_STRING,))
    }

    default_text = _("Type here to search...")

    # The font style of the text in the entry.
    #font_style = None

    # TODO: What is this?
    search_timeout = 0

    def __init__(self, accel_group = None):
        gtk.Entry.__init__(self)

        self.set_width_chars(30)
        self.set_text(self.default_text)
        self.set_size_request(-1, 32)
        self.connect("changed", lambda w: self._queue_search())
        self.connect("focus-in-event", self._entry_focus_in)
        self.connect("focus-out-event", self._entry_focus_out)
        #self.connect("icon-press", self._icon_press)
        self.show_all()

    def _icon_press(self, widget, pos, event):
        # Note: GTK_ENTRY_ICON_SECONDARY does not seem to be bound in PyGTK.
        if int(pos) == 1 and not self.get_text() == self.default_text:
            self._entry_clear_no_change_handler()

    def _entry_focus_in(self, widget, x):
        if self.get_text() == self.default_text:
            self.set_text("")
            #self.modify_font(self.font_style)

    def _entry_focus_out(self, widget, x):
        if self.get_text() == "":
            self.set_text(self.default_text)
            #self.modify_font(self.font_style)

    def _entry_clear_no_change_handler(self):
        if not self.get_text() == self.default_text:
            self.set_text("")

    def _queue_search(self):
        if self.search_timeout != 0:
            gobject.source_remove(self.search_timeout)
            self.search_timeout = 0

        if self.get_text() == self.default_text or len(self.get_text()) == 0:
            self.emit("clear")
        else:
            self.search_timeout = gobject.timeout_add(200, self._typing_timeout)

    def _typing_timeout(self):
        if len(self.get_text()) > 0:
            self.emit("search", self.get_text())

        self.search_timeout = 0
        return False



lib_applet = TopicsWindow()

